//Sample program for MSP430-5438STK
//Test LCD Display
//Slide on the display the text "www.olimex.com/dev"

#include "io430.h"
#include "font.h"

#define OSC_FREQ 18000000
#define LCD_FREQ  1000000

#if   defined  (TLS8204)
#define CONTRAST        0x5A
#elif defined  (PCD8544)
#define CONTRAST        0x7F
#endif

//LCD definitions +
#define SEND_CMD 0
#define SEND_CHR 1
#define LCD_X_RES                  84
#define LCD_Y_RES                  48
#define LCD_START_LINE_ADDR	(66-2)
//LCD definitions -

unsigned char text[22] = "www.olimex.com/dev    ", LCDMemory[504], inverse=1;
int index=0;

//prototypes
void SPI2_Init(unsigned long freq);
void SPI2_Transmit(unsigned char data);
unsigned char SPI2_Receive();
void LCD_Init();
void LCD_Send(unsigned char data, unsigned char cd);
void LCD_Clear();
void LCD_Update(unsigned char mode, unsigned char num);
void LCD_Contrast(unsigned char contrast);
void LCD_Clear_Memory();
void LCD_Set_Position(unsigned char x, unsigned int y);

void Delay(unsigned long time)
{
  while (time--);
}

void CLOCK_XT2_Init()
{
	int timeout = 5000;
	P11DIR = P11DIR | 0x02 | 0x04;            //set P11.1 & P11.2 as output
	P11SEL = P11SEL | 0x02 | 0x04;            //set P11.1 & P11.2 as MCLK/SMCLK function
     P5SEL = P5SEL | 0x04 | 0x08;              //set P5.2 & P5.3 as XT2IN/XT2OUT

     UCSCTL6 = UCSCTL6 & (~XT2OFF);            //set XT2OFF=0 (bit8 = 0) --> enable XT2
     UCSCTL3 = UCSCTL3 | 0x20;                 //set SELREF = 010 --> FLL clock source = REFOCLK
     UCSCTL4 = UCSCTL4 | 0x200;                //set SELA = 010 --> ACLK=REFO

     do
     {
       if (!(UCSCTL7 & 0x08)) break;
       UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
       SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
       timeout--;
     }
     while ((SFRIFG1 & 0x02) && timeout);      // Test oscillator fault flag

	UCSCTL5 = UCSCTL5 | 0x0005;               //MCLK/SMCLK divisor = 32

     UCSCTL4 = UCSCTL4 | 0x55;                 //SMCLK=MCLK=XT2
     UCSCTL4 = ((UCSCTL4 & ~0x100) & ~0x200) & ~0x400;     //ACLK=XT1  (SELA = 0b000)
}


//SPI Functions +

    void SPI2_Init(unsigned long freq)
    {
		
      //SPI & LCD PORTS
      P9SEL = P9SEL | 0x0E;                   //P9.1/P9.2/P9.3 set as alternate function
      P9DIR = P9DIR | 0x0B;                   //P9.0/P9.1/P9.3 set as output
      P9DIR = P9DIR | 0xC0;                   //LCD Reset, LCD data/command bits set as output
	
      unsigned int prescaler;
      UCB2CTL1 = UCB2CTL1 | 0x01;         //UCSWRST = 1 (reset state enable)

      //set format:
      //set clock polarity and phase (UCCKPH = 0; UCCKPL = 1) --> bit7=0; bit6=1
      //MSB first; bit0 (UCSYNC) = 1; bit3 (master mode) = 1; --> bit5=1; bit4=0; bit3 =3; bit0=1;
      //set mode: 3pin SPI --> bit1=0; bit2=3;
      //UCB2CTL0 = 0b01101001 = 0x69
      UCB2CTL0 = 0x69;

      UCB2CTL1 = UCB2CTL1 | 0x80;                     //clock source - SMCLK

      prescaler = OSC_FREQ / freq;
      UCB2BR0 = prescaler % 256;
      UCB2BR1 = prescaler / 256;

      P9SEL = P9SEL | 0x0E;               //P9.0/P9.1/P9.2/P9.3 set as alternate function
      P9DIR = P9DIR | 0x0B;               //P9.0/P9.1/P9.3 set as output

      UCB2CTL1 = UCB2CTL1 & ~(0x01);       //UCSWRST = 0 (reset state disable)
    }

    unsigned char SPI2_Receive()
    {
      if (UCB2IFG & 0x01)
        return UCB2RXBUF;
      else
        return 0;
    }

    void SPI2_Transmit(unsigned char data)
    {
      while (!(UCB2IFG & 0x02));
      UCB2TXBUF = data;
    }

//SPI Functions -

//LCD Functions +

    void LCD_Init()
    {
      P9OUT = P9OUT & 0xBF;         //LCD_RES = 0   //~(1<<6)
      Delay(1000);
      P9OUT = P9OUT | 0x40;         //LCD_RES = 1   //1<<6
      Delay(1000);

      //from Dimitar's PIC source +
        //Send sequence of command +
          LCD_Send(0x21, SEND_CMD);  // LCD Extended Commands.
          //Extended commands +
            LCD_Send(0xC8, SEND_CMD);  // Set Contrast.
            LCD_Send(0x04 | !!(LCD_START_LINE_ADDR&(0x40)), SEND_CMD);      // Set Temp S6 for start line
            LCD_Send(0x40 | (LCD_START_LINE_ADDR & ((0x40)-1)), SEND_CMD);  // Set Temp S[5:0] for start line
            LCD_Send(0x12, SEND_CMD);  // LCD bias mode 1:68.
          //Extended commands -
          LCD_Send(0x20, SEND_CMD);  // LCD Standard Commands, Horizontal addressing mode.
          LCD_Send(0x08, SEND_CMD);  // LCD blank
          LCD_Send(0x0C, SEND_CMD);  // LCD in normal mode.
        //Send sequence of command -
      //from Dimitar's PIC source -

      //Optional commands +
        LCD_Contrast(CONTRAST);       //Increase contrast
        LCD_Clear();              //Clear Screen
        LCD_Set_Position(0, 0);   //0, 0 coordinates of the display
      //Optional Commands -
    }

    void LCD_Send(unsigned char data, unsigned char cd)
    {
      P9OUT = P9OUT & ~(0x01);

      if (cd == SEND_CHR)         //check for sending char/command
        P9OUT = P9OUT | 0x80;     //sending char                    //1<<7
      else
        P9OUT = P9OUT & 0x7F;     //sending command                 //~(1<<7)

      SPI2_Transmit(data);        //transmitting char/command

      P9OUT = P9OUT | 0x01;
    }

    void LCD_Clear_Memory()
    {
      for (index=0; index<504; index++)
        LCDMemory[index] = 0;
      index = 0;
    }

    /*
      mode indicates inverse display:
      (shows black pixels as white and white pixels as black)

      the 1 in binary num indicates which of the lines
      must/must not be inverted
      ex. num = 13 --> 0b001101 --> lines 0, 2, 3 will be inverted
    */
    void LCD_Update(unsigned char mode, unsigned char num)
    {
      char x, y;
      for(y=0; y<48/8; y++)
      {
        LCD_Send(0x80, SEND_CMD);     //x - home
        LCD_Send(0x40|y, SEND_CMD);   //y - next line

        if (((mode==inverse) && !(num & 1<<y)) || ((mode != inverse) && (num & 1<<y)))
          for(x=0; x<84; x++)
            LCD_Send(LCDMemory[y*84+x], SEND_CHR);
        else
          for(x=0; x<84; x++)
            LCD_Send(~LCDMemory[y*84+x], SEND_CHR);
      }
    }

    void LCD_Clear()
    {
      LCD_Clear_Memory();
      LCD_Update(1, 0x00);
    }

    void LCD_Contrast(unsigned char contrast)
    {
        LCD_Send(0x21, SEND_CMD);
        //LCD Extended Commands +
          LCD_Send(0x80 | contrast, SEND_CMD);  // Set LCD Vop (Contrast).
        //LCD Extended Commands -
        LCD_Send(0x20, SEND_CMD);         //LCD Standard Commands, horizontal addressing mode
    }

    void LCD_Set_Position(unsigned char x, unsigned int y)
    {
      //Optional calculations +
        //in case the position is out of the screen:
        //if X > 83 it goes to the next (or more) line.
        //if Y > 5 it return to the 0, 1, 2... line
          y = y + x/84;
          y = y % 6;
          x = x % 84;
      //Optional calculations -
      index = 84*y + x;

      //send cursor to x, y coordinates +
        LCD_Send(0x80 | x, SEND_CMD);
        LCD_Send(0x40 | y, SEND_CMD);
      //send cursor to x, y coordinates -
    }

    void LCD_Write(unsigned char *a, int l)
    {
      unsigned char i;
      while (*a && l)
      {
	   l--;
        for (i=0; i<5; i++)
          LCDMemory[index++] = FontLookup[*a-32][i];
        LCDMemory[index++] = 0x00;    //space between signs
        a++;
      }
    }

//LCD Functions -


int main( void )
{
  unsigned int i;
  unsigned char c;
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;

  CLOCK_XT2_Init();
  SPI2_Init(LCD_FREQ);
  LCD_Init();



  while (1)
  {
	c = text[0];
	for (i=0; i<22; i++) text[i] = text[i+1];
	text[21] = c;
	LCD_Clear_Memory();
  	LCD_Write(text, 14);
  	LCD_Update(1, 0);
	for (i=30000; i; i--);
  }
}
